{-# LANGUAGE TypeApplications #-}
{-# LANGUAGE OverloadedStrings #-}
{--|
Module      : Main.hs
Description : Main for Gargantext Tasty Tests
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}

module Main where

import Gargantext.Prelude hiding (isInfixOf)

import qualified Test.Core.LinearAlgebra         as LinearAlgebra
import qualified Test.Core.Notifications         as Notifications
import qualified Test.Core.Orchestrator          as Orchestrator
import qualified Test.Core.Similarity            as Similarity
import qualified Test.Core.Text.Corpus.Query     as CorpusQuery
import qualified Test.Core.Text.Corpus.TSV       as TSVParser
import qualified Test.Core.Utils                 as Utils
import qualified Test.Core.Worker                as Worker
import qualified Test.Graph.Clustering           as Clustering
import qualified Test.Graph.Distance             as Distance
import qualified Test.Ngrams.Lang.Occurrences    as Occurrences
import qualified Test.Ngrams.NLP                 as NLP
import qualified Test.Ngrams.Query               as NgramsQuery
import qualified Test.Ngrams.Terms               as NgramsTerms
import qualified Test.Offline.Errors             as Errors
import qualified Test.Offline.JSON               as JSON
import qualified Test.Offline.Ngrams             as Ngrams
import qualified Test.Offline.Phylo              as Phylo
import qualified Test.Offline.Stemming.Lancaster as Lancaster
import qualified Test.Parsers.Date               as PD
import qualified Test.Utils.Crypto               as Crypto
import qualified Test.Utils.Jobs                 as Jobs
import qualified Test.API                        as API
import qualified Test.Database.Operations        as DB
import qualified Test.Database.Transactions      as DBT
import qualified Test.Server.ReverseProxy        as ReverseProxy

import Data.Text (isInfixOf)
import Data.Text qualified as T
import System.IO (hGetBuffering, hSetBuffering)
import System.Posix.Process
import System.Posix.Signals
import System.Process
import Test.Tasty
import Test.Tasty.Hspec
import qualified Prelude

-- | https://mercurytechnologies.github.io/ghciwatch/integration/tasty.html
protectStdoutBuffering :: IO a -> IO a
protectStdoutBuffering action =
  bracket
    (hGetBuffering stdout)
    (\bufferMode -> hSetBuffering stdout bufferMode)
    (const action)

startCoreNLPServer :: IO ProcessHandle
startCoreNLPServer = do
  putText "calling start core nlp"
  devNull <- openFile "/dev/null" WriteMode
  let p = proc "startCoreNLPServer.sh" []
  (_, _, _, hdl) <- (createProcess $ p { cwd = Nothing
                    -- NOTE(adn) Issue #451, this one has to stay disabled, because if we
                    -- turn it on, despite the confusing documentation on the `process` library
                    -- it will cause the Haskell RTS to completely ignore the Ctrl^c and instead
                    -- delegate it exclusively to the process here, which means that our CoreNLP
                    -- server will shut down correctly, but the test running will stop responding
                    -- to Ctrl^C requests.
                    , delegate_ctlc = False
                    , create_group = True
                    , std_out = UseHandle devNull
                    , std_err = UseHandle devNull
                    }) `catch` \e -> case e of
                                           _ | True <- "does not exist" `isInfixOf` (T.pack . show @SomeException $ e)
                                             -> Prelude.fail $ "Cannot execute the 'startCoreNLPServer.sh' script. Make sure you are in a nix environment."
                                             | otherwise -> throwIO e
  pure hdl

killProcessTree :: ProcessHandle -> IO ()
killProcessTree ph = do
  pid <- getPid ph
  case pid of
    Nothing -> putText "Process already terminated"
    Just p -> do
      pgid <- getProcessGroupIDOf p
      signalProcessGroup keyboardSignal pgid

main :: IO ()
main = do
  utilSpec         <- testSpec "Utils" Utils.test
  clusteringSpec   <- testSpec "Graph Clustering" Clustering.test
  distanceSpec   <- testSpec "Graph Distance" Distance.test
  dateSplitSpec    <- testSpec "Date split" PD.testDateSplit
  cryptoSpec       <- testSpec "Crypto" Crypto.test
  nlpSpec          <- testSpec "NLP" NLP.test
  jobsSpec         <- testSpec "Jobs" Jobs.test
  similaritySpec   <- testSpec "Similarity" Similarity.test
  asyncUpdatesSpec <- testSpec "Notifications" Notifications.test
  occurrencesSpec  <- testSpec "Occurrences" Occurrences.test
  apiSpec          <- testSpec "API" API.tests
  revProxySpec     <- testSpec "Reverse Proxy" ReverseProxy.tests
  dbSpec           <- testSpec "Database Operations" DB.tests
  dbTxSpec         <- testSpec "Database Transactions" DBT.tests
  nodeStorySpec    <- testSpec "Node Story" DB.nodeStoryTests

  protectStdoutBuffering $ defaultMain $
    withResource startCoreNLPServer killProcessTree $ \_ -> testGroup "Gargantext"
      [ utilSpec
      , clusteringSpec
      , distanceSpec
      , dateSplitSpec
      , cryptoSpec
      , nlpSpec
      , jobsSpec
      , occurrencesSpec
      , NgramsQuery.tests
      , occurrencesSpec
      , CorpusQuery.tests
      , TSVParser.tests
      , JSON.tests
      , Ngrams.tests
      , Errors.tests
      , similaritySpec
      , Phylo.tests
      , testGroup "Stemming" [ Lancaster.tests ]
      , Worker.tests
      , asyncUpdatesSpec
      , Notifications.qcTests
      , Orchestrator.qcTests
      , NgramsTerms.tests
      , LinearAlgebra.tests
      , apiSpec
      , revProxySpec
      , dbSpec
      , dbTxSpec
      , nodeStorySpec
      ]
